span.rst-current-version{
	display: none !important
}

td{
	white-space: normal !important;
	word-break: break-word !important;
}

.rst-content .highlight,
.rst-content pre{
	border: none !important;
	background: none !important;
	padding: 0 !important;
	box-shadow: none !important;
}

.rst-content pre code.hljs{
	display: block;
	padding: 0.75rem;
	border-radius: 4px;
	background: #eeffcc !important;
}

/* Style tabbed content blocks generated by pymdownx.tabbed. */
.tabbed-set{
	display: flex;
	flex-wrap: wrap;
	margin: 1rem 0;
	border: 1px solid #d4d9de;
	border-radius: 4px;
	overflow: hidden;
	background: #ffffff;
}

.tabbed-set > input{
	position: absolute;
	opacity: 0;
	pointer-events: none;
}

.tabbed-set > label{
	order: 1;
	padding: 0.65rem 1rem;
	cursor: pointer;
	font-weight: 600;
	background: #f3f5f7;
	border-bottom: 2px solid transparent;
	border-right: 1px solid #d4d9de;
}

.tabbed-set > label:hover{
	background: #e8edf1;
}

.tabbed-set > .tabbed-content{
	order: 99;
	width: 100%;
	display: none;
	padding: 1rem;
	border-top: 1px solid #d4d9de;
	background: #ffffff;
}

.tabbed-set > input:checked + label{
	background: #ffffff;
	border-bottom-color: #2b7cb8;
}

.tabbed-set > input:checked + label + .tabbed-content{
	display: block;
}

@media (max-width: 768px){
	.tabbed-set > label{
		flex: 1 1 50%;
		text-align: center;
	}

	.tabbed-set > .tabbed-content{
		padding: 0.75rem;
	}
}
