#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.pitcairn_islands import PitcairnIslands
from tests.common import CommonCountryTests


class TestPitcairnIslands(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(PitcairnIslands)

    def test_new_years_day(self):
        name = "New Year's Day"
        # Public Holidays.
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        self.assertNoHolidayName(name, (f"{year}-01-02" for year in self.full_range))
        # Government Holidays.
        self.assertGovernmentHolidayName(name, (f"{year}-01-02" for year in self.full_range))

    def test_bounty_day(self):
        self.assertHolidayName("Bounty Day", (f"{year}-01-23" for year in self.full_range))

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_queens_birthday(self):
        name = "Queen's Birthday"
        self.assertHolidayName(
            name,
            "2016-06-11",
            "2017-06-10",
            "2018-06-09",
            "2019-06-08",
            "2020-06-13",
            "2021-06-12",
            "2022-06-11",
        )
        self.assertNoHolidayName(name, range(2023, self.end_year))

    def test_kings_birthday(self):
        name = "King's Birthday"
        self.assertHolidayName(
            name,
            "2023-06-05",
            "2024-06-03",
            "2025-06-02",
        )
        self.assertHolidayName(name, range(2023, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2023))

    def test_pitcairn_day(self):
        self.assertHolidayName("Pitcairn Day", (f"{year}-07-02" for year in self.full_range))

    def test_christmas_day(self):
        self.assertHolidayName("Christmas Day", (f"{year}-12-25" for year in self.full_range))

    def test_boxing_day(self):
        self.assertHolidayName("Boxing Day", (f"{year}-12-26" for year in self.full_range))

    def test_anzac_day(self):
        name = "ANZAC Day"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(name, (f"{year}-04-25" for year in self.full_range))

    def test_remembrance_day(self):
        name = "Remembrance Day"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(name, (f"{year}-11-11" for year in self.full_range))

    def test_2022_public(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "New Year's Day"),
            ("2022-01-23", "Bounty Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-06-11", "Queen's Birthday"),
            ("2022-07-02", "Pitcairn Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
        )

    def test_2022_government(self):
        self.assertGovernmentHolidaysInYear(
            2022,
            ("2022-01-02", "New Year's Day"),
        )

    def test_2022_workday(self):
        self.assertWorkdayHolidaysInYear(
            2022,
            ("2022-04-25", "ANZAC Day"),
            ("2022-11-11", "Remembrance Day"),
        )
